<!doctype html>
    <html lang="en">
    <head>
      <meta charset="utf-8">
      <title>Navigator API</title>
      <meta name="description" content="Navigator API">
      <meta name="author" content="Navigator API">
      <link rel="stylesheet" href="static/css/styles.css?v=1.0">
  </head>
  <body>
      <script src="static/js/scripts.js"></script>
      <h1>Welcome to Navigator!</h1>
      <p>This is <b>Navigator</b>, a Framework with "batteries-included",
        a comprehensive collection of asyncio-based libraries to easily create any python project in minutes.
      </p>
      <p>
        Navigator is based on python >= 3.8, Asyncio and other fantastic asynchronous technologies like:
        <ul>
          <li> Aiohttp </li>
          <li> Python 3.8 </li>
          <li> Asyncio+uvloop </li>
          <li> asyncpg </li>
          <li> SockJS </li>
        </ul>
        And many others, a Framework designed with a "functional-first" philosophy,
        is an async "micro-Django" with many tools for easy deployment
        and development of async applications.
        </p>
        <p>
          Navigator has some fantastic features as:
          <ul>
            <li> <b>Applications:</b> like django, you can create "apps" in the "apps/" folder, extending from AppConfig, and Navigator make the rest. </li>
            <li> <b>Sub-Domains:</b> an Application can start its own SubDomain, perfect for isolating applications between tenants. </li>
            <li> <b>Central Config:</b> You can declare your configuration in INI files, secrets in environment files, but everything will be access from one single point of truth. </li>
            <li> <b>Auto-Connections:</b> Based on AsyncDB (a tool for integrating several database connectors), every App can start automatically a connection, accessed by "request" object. </li>
            <li> <b>Authentication Backend with Middlewares:</b> A pluggable Authentication/Authorization Backend with Middleware Support. </li>
            <li> <b>Built-in Notifications:</b> Can send Notifications and messages with several backends (emails, sms, telegram, slack) using background jobs. </li>
            <li> And other features: WebSockets, Jinja Templates, Model Views, etc </li>
          </ul>
          </p>
  </body>
</html>
