"""Navigator Meta information."""

__title__ = 'navigator-api'
__description__ = ('Navigator Web Framework based on aiohttp, '
                   'with batteries included.')
__version__ = '2.0.8'
__author__ = 'Jesus Lara'
__author_email__ = 'jesuslarag@gmail.com'
__license__ = 'BSD'
